#include <stdio.h>
#include <stdlib.h>

#define ERROR -99999999

typedef struct {
	    int *array;	/* Zeiger auf das erste Element des Arrays */
	    int size;	/* Groesse des allokierten Speichers */
	    int used;	/* tatsaechliche Groesse des Arrays; used <= size! */
	} Array;

/*
 * Neues Integer-Array mit der angegebenen
 * Groesse erzeugen und zurueck geben
 *
 * Parameter "Array **a" notwendig, um Array-Zeiger
 * call by reference zu uebergeben!
 */
void arrayNew(Array **a, int len) {
    *a = (Array*) calloc(1, sizeof(Array));
    (*a)->size = (*a)->used = len;
    (*a)->array = (int*) calloc(len, sizeof(int));
}

/*
 * Integer-Array wieder freigeben
 */
void arrayDelete(Array **a) {
    free((*a)->array);
    (*a)->array = NULL;
    free(*a);
    *a = NULL;
}

/*
 * Groesse des Integer-Arrays veraendern
 */
void arrayResize(Array *a, int len) {
    if (a->size < len) {
	a->array = (int*) realloc(a->array, sizeof(int) * len);
	a->size = a->used = len;
    }
    else
	a->used = len;
}

/*
 * Laenge/Groesse des Arrays ermitteln
 */
int arrayLength(Array *a) {
    return a->used;
}

/*
 * Element des Arrays lesen
 */
int arrayGet(Array *a, int i) {
    if (i < 0 || i >= a->used)
	return ERROR;	/* :-( */
    return a->array[i];
}

/*
 * Element des Arrays neu setzen
 */
void arraySet(Array *a, int i, int value) {
    if (i >= 0 && i < a->used)
	a->array[i] = value;
}

int main() {
    int i;
    Array *a;
    
    arrayNew(&a, 5);

    for (i = 0; i < arrayLength(a); i++)
	arraySet(a, i, i);

    for (i = 0; i < arrayLength(a); i++)
	printf("a[%d] = %d\n", i, arrayGet(a, i));

    puts("-----------");

    arrayResize(a, 3);
    
    for (i = 0; i < arrayLength(a); i++)
	printf("a[%d] = %d\n", i, arrayGet(a, i));

    puts("-----------");

    arrayResize(a, 7);
    arraySet(a, 5, 10);
    
    for (i = 0; i < arrayLength(a); i++)
	printf("a[%d] = %d\n", i, arrayGet(a, i));

    arrayDelete(&a);
    
    return 0;    
}
